@@ -2,7 +2,7 @@ package Ukigumo::Client;
 use strict;
 use warnings;
 use 5.008001;
-our $VERSION = '0.13';
+our $VERSION = '0.14';
 
 use Carp ();
 use Capture::Tiny;
@@ -19,6 +19,7 @@ use JSON qw(decode_json);
 use File::Temp;
 use File::HomeDir;
 use URI::Escape qw(uri_escape);
+use YAML::Tiny;
 
 use Ukigumo::Constants;
 
@@ -117,13 +118,30 @@ sub run {
             return;
         }
         my $vc_log = $self->vc->get_log($orig_revision, $current_revision);
-		$self->log('run executor : ' . ref $self->executor);
-        my $status = $self->executor->run($self);
+
+        my $conf = $self->load_config();
+
+        $self->run_commands($conf, 'before_install');
+
+        $self->install($conf);
+
+        $self->run_commands($conf, 'before_script');
+
+        my $executor = defined($conf->{script}) ? Ukigumo::Client::Executor::Command->new(command => $conf->{script}) : $self->executor;
+
+		$self->log('run executor : ' . ref $executor);
+        my $status = $executor->run($self);
+
 		$self->log('finished testing : ' . $status);
 
-        my ($report_url, $last_status) = $self->send_to_server($status, $current_revision, $vc_log);
+        $self->run_commands($conf, 'after_script');
+
+        my ($report_url, $last_status) = $self->send_to_server(
+            $status, $current_revision, $vc_log
+        );
 
         $self->log("sending notification: @{[ $self->branch ]}, $status");
+        $self->_load_notifications($conf);
         for my $notify (@{$self->notifiers}) {
             $notify->send($self, $status, $last_status, $report_url);
         }
@@ -132,6 +150,68 @@ sub run {
     $self->log("end testing");
 }
 
+sub _load_notifications {
+    my ($self, $conf) = @_;
+    for my $type (keys %{$conf->{notifications}}) {
+        if ($type eq 'ikachan') {
+            my $c = $conf->{notifications}->{$type};
+               $c = [$c] unless ref $c eq 'ARRAY';
+
+            for my $args (@$c) {
+                my $notifier = Ukigumo::Client::Notify::Ikachan->new($args);
+                push @{$self->{notifiers}}, $notifier;
+            }
+        } else {
+            die "Unknown notification type: $type";
+        }
+    }
+}
+
+sub load_config {
+    my $self = shift;
+
+    if ( -e '.ukigumo.yml' ) {
+        my $y = eval { YAML::Tiny->read('.ukigumo.yml') };
+        $self->log("Bad .ukigumo.yml: $@") if $@;
+        $y ? $y->[0] : undef;
+    }
+    else {
+        $self->log("There is no .ukigumo.yml");
+        undef;
+    }
+}
+
+# Install deps
+sub install {
+    my ($self, $conf) = @_;
+
+    my $install = do {
+        if ($conf->{install}) {
+            $conf->{install};
+        } else {
+            if (-f 'Makefile.PL' || -f 'cpanfile' || -f 'Build.PL') {
+                'cpanm --notest --installdeps .';
+            } else {
+                undef;
+            }
+        }
+    };
+    if ($install) {
+        $self->log("[install] $install");
+        system($install)
+            == 0 or die "Failure in installing: $install";
+    }
+}
+
+sub run_commands {
+    my ($self, $yml, $phase) = @_;
+    for my $cmd (@{$yml->{$phase} || []}) {
+        $self->log("[${phase}] $cmd");
+        system($cmd)
+            == 0 or die "Failure in ${phase}: $cmd";
+    }
+}
+
 sub send_to_server {
     my ($self, $status, $current_revision, $vc_log) = @_;
 
